#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pyformatters_bel_table']

package_data = \
{'': ['*']}

install_requires = \
['pymultirole_plugins>=0.5.0,<0.6.0',
 'pandas==1.2.3',
 'openpyxl==3.0.7',
 'pytest']

entry_points = \
{'pyformatters.plugins': ['bel_table = '
                          'pyformatters_bel_table.bel_table:BELTableFormatter']}

setup(name='pyformatters-bel_table',
      version='0.5.50',
      description='Sherpa BELTable formatter',
      author='Olivier Terrier',
      author_email='olivier.terrier@kairntech.com',
      url='https://kairntech.com/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.8',
     )
